<?php

/**
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or (at
 *   your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful, but
 *   WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *   General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, see <http://www.gnu.org/licenses/>.
 *
 *   @author          Administrator
 *   @copyright       2015, Administrator
 *   @link            http://blackcat-cms.org
 *   @license         http://www.gnu.org/licenses/gpl.html
 *   @category        CAT_Tools
 *   @package         Image Optimizer
 *
 */

if (defined('CAT_PATH')) {
    if (defined('CAT_VERSION')) include(CAT_PATH.'/framework/class.secure.php');
} elseif (file_exists($_SERVER['DOCUMENT_ROOT'].'/framework/class.secure.php')) {
    include($_SERVER['DOCUMENT_ROOT'].'/framework/class.secure.php');
} else {
    $subs = explode('/', dirname($_SERVER['SCRIPT_NAME']));    $dir = $_SERVER['DOCUMENT_ROOT'];
    $inc = false;
    foreach ($subs as $sub) {
        if (empty($sub)) continue; $dir .= '/'.$sub;
        if (file_exists($dir.'/framework/class.secure.php')) {
            include($dir.'/framework/class.secure.php'); $inc = true;    break;
        }
    }
    if (!$inc) trigger_error(sprintf("[ <b>%s</b> ] Can't include class.secure.php!", $_SERVER['SCRIPT_NAME']), E_USER_ERROR);
}

if ( ! class_exists( 'Image', false ) ) {
    require_once CAT_PATH.'/modules/lib_images/inc/class.Images.php';
}

function imageOptimizer(&$content)
{
    $changes  = 0;
    $settings = _imageoptimizer_settings();
    if(!isset($settings['is_active']) || $settings['is_active'] !== 'y') return;

    $dom      = new DOMDocument();
    libxml_use_internal_errors(true);
    if ( false !== $dom->loadHTML($content, LIBXML_NOERROR | LIBXML_NOWARNING) )
    {
        $images = $dom->getElementsByTagName('img');
        if(count($images))
        {
            $folder   = $settings['folder'];
            foreach($images as $img)
            {
                if (! $img->getAttribute('class') || stripos($img->getAttribute('class'),$settings['ignore_class']) === false) {
                    $src  = $img->getAttribute('src');
                    $file = CAT_Helper_Validate::uri2path($src);
                    if(file_exists($file))
                    {
                        $path     = pathinfo($file,PATHINFO_DIRNAME);
                        $filename = pathinfo($file,PATHINFO_FILENAME);
                        $suffix   = pathinfo($file,PATHINFO_EXTENSION);
                        // check css
                        $css      = $img->getAttribute('style');
                        $style    = array();
                        if($css)
                        {
                            if(preg_match_all('~;?(width|height)\:\s?([^;].+?)px;?~i',$css,$m,PREG_SET_ORDER))
                            {
                                foreach($m as $i => $r)
                                {
                                    $style[$r[1]] = $r[2];
                                }
                            }
                        }
                        if(is_array($style) && isset($style['width']) && isset($style['height']))
                        {
                            $min_file = CAT_Helper_Directory::sanitizePath($path.'/'.$folder.'/'.$filename.'_'.$style['width'].'_'.$style['height'].'.'.$suffix);
                            if(!file_exists($min_file))
                            {
                                $image       = new Image($file);
                                $real_height = $image->getHeight();
                                $real_width  = $image->getHeight();
                                if($real_height > $style['height'] || $real_width > $style['width'])
                                {
                                    if(!is_dir($path.'/'.$folder)) CAT_Helper_Directory::createDirectory($path.'/'.$folder);
                                    $image->resize(
                                        $style['width'],
                                        $style['height'],
                                        "fit","c","c",
                                        (
                                            (   isset($settings['jpg_quality'])
                                             && is_numeric($settings['jpg_quality'])
                                             && $settings['jpg_quality'] <= 100
                                             && $settings['jpg_quality'] > 40
                                            ) ? $settings['jpg_quality'] : 90
                                        )
                                    );
                                    $image->save($filename.'_'.$style['width'].'_'.$style['height'],$path.'/'.$folder,$suffix);
                                }
                            }
                            $img->setAttribute('src',_imageoptimier_path2uri($min_file));
                            $changes++;
                        }
                    }
                }
            }
        }
        if($changes) $content = $dom->saveHTML();
    }
    else
    {
        echo "Image optimization error<br />";
    }
}

function _imageoptimier_path2uri($path)
{
    return str_ireplace(CAT_Helper_Directory::sanitizePath(CAT_PATH),CAT_Helper_Validate::sanitize_url(CAT_Helper_Validate::getURI(CAT_URL)),CAT_Helper_Directory::sanitizePath($path));
}
function _imageoptimizer_settings()
{
    $db       = CAT_Helper_DB::getInstance();
    $result   = $db->query('SELECT * FROM `:prefix:mod_bcimageoptimizer_settings`')->fetchAll();
    $settings = array();
    foreach($result as $item)
        $settings[$item['set_name']] = $item['set_content'];
    return $settings;
}